import { ComponentClass } from 'react';
import Taro, { Component, Config } from '@tarojs/taro';
import { View, Image } from '@tarojs/components';

import scanIcon from '../../images/icon_s@2x.png';

import './index.scss';
import { registerAndLogin } from '../../api/customer';
import { getDeviceConfig } from '../../api/device';
import { UserState, updateUserInfo } from '../../store/rootReducers/userinfo';
import { connect } from '@tarojs/redux';
import { Customer } from '../../types/Customer/Customer';
import {
  DeviceState,
  updateDeviceData,
  resetDeviceData,
} from '../../store/rootReducers/device';
import PrepayConfig from '../../types/Order/Order';
import { updatePayData } from '../../store/rootReducers/prepayConfig';

type DeviceEntity = {
  serviceId: number;
  equipmentNum: string;
};

type PageStateProps = {
  userinfo: Customer;
};

type PageDispatchProps = {
  updateUserInfo: (e: UserState) => void;
  updateDeviceData: (e: DeviceState) => void;
  updatePayData: (e: PrepayConfig[]) => void;
  resetDeviceData: () => void;
};

type PageOwnProps = {};

type PageState = {
  errorText: string;
};

type IProps = PageStateProps & PageDispatchProps & PageOwnProps;

interface Index {
  props: IProps;
  state: PageState;
}

@connect(
  ({ userinfo }) => ({
    userinfo,
  }),
  dispatch => ({
    updateUserInfo(data: UserState) {
      dispatch(updateUserInfo(data));
    },
    updateDeviceData(data: DeviceState) {
      dispatch(updateDeviceData(data));
    },
    updatePayData(data: PrepayConfig[]) {
      dispatch(updatePayData(data));
    },
    resetDeviceData() {
      dispatch(resetDeviceData());
    },
  }),
)
class Index extends Component {
  config: Config = {
    navigationBarTitleText: '多彩自助服务',
  };

  constructor(props) {
    super(props);
    this.state = {
      errorText: '',
    };
  }

  componentWillMount() {
    this.loginHandle().then(() => {
      let queryArr = Object.keys(this.$router.params);
      console.log(this.$router.params);

      let scene: string = queryArr.length > 0 ? queryArr[0] : '';
      console.log('scene:', scene);

      if (scene) {
        Taro.showLoading();
        let entity = this.getDeviceEntity(scene);
        if (entity) {
          this.getDeviceConfigHandle(entity);
        }
      } else {
        Taro.redirectTo({
          url: '/pages/scanEnter/scanEnter',
        });
      }
    });
  }

  getDeviceEntity(paramStr: string): DeviceEntity {
    let serviceId = Number(paramStr.slice(0, 2));
    let equipmentNum = paramStr.slice(2);
    return {
      serviceId,
      equipmentNum,
    };
  }

  loginHandle() {
    const { updateUserInfo } = this.props;

    return Taro.login().then(res => {
      const { code } = res;
      return registerAndLogin({
        code,
        // userName: '',
      }).then(res => {
        const { token, customerId } = res;
        updateUserInfo({
          token,
          customerId,
        });
      });
    });
  }

  scanHandle() {
    Taro.scanCode({
      onlyFromCamera: true,
      scanType: ['qrCode'],
    })
      .then(res => {
        console.log(res);
        const { path, result } = res;
        if (path) {
          let queryArr = path.split('?');
          let queryStr = queryArr.length >= 2 ? queryArr[1] : '';
          console.log(result, queryArr, queryStr);
          const entity = this.getDeviceEntity(queryStr);
          this.getDeviceConfigHandle(entity);
        } else {
          Taro.showToast({
            title: '请扫描正确的小程序',
            icon: 'none',
          });
          this.setState({
            errorText: '请扫描正确的小程序',
          });
        }
      })
      .catch(err => {
        console.error(err);
      });
  }

  getDeviceConfigHandle({ equipmentNum, serviceId }: DeviceEntity) {
    Taro.showLoading();
    const {
      userinfo,
      updateDeviceData,
      updatePayData,
      resetDeviceData,
    } = this.props;
    return getDeviceConfig({
      customerId: userinfo.customerId,
      equipmentNum: equipmentNum,
      serviceId: serviceId,
    })
      .then(res => {
        Taro.hideLoading();
        console.log(res);
        const { deviceInfoResponse, prepayConfigs } = res;
        if (deviceInfoResponse.isUsed) {
          console.log('设备使用中');
          return;
        }
        if (deviceInfoResponse) {
          updateDeviceData(deviceInfoResponse);
          updateDeviceData({originCode: equipmentNum});
        } else {
          resetDeviceData();
        }
        if (prepayConfigs) {
          updatePayData(prepayConfigs);
        } else {
          updatePayData([]);
        }
        Taro.redirectTo({
          url: '/pages/pay/pay',
        });
      })
      .catch(err => {
        console.error(err);
        Taro.showToast({
          title: err.msg || '请扫描正确的设备码',
          icon: 'none',
        });
        this.setState({
          errorText: err.msg || '请扫描正确的设备码',
        });
      });
  }

  getUserInfoHandle(res) {
    console.log(res);
    const { detail } = res;
    if (detail.userInfo) {
      // this.scanHandle(detail.userInfo);
    }
  }

  clickHandle() {
    this.scanHandle();
  }

  render() {
    const { errorText } = this.state;
    return (
      <View className='index'>
        {errorText}
        {/* <Button
          className='scan-icon'
          onGetUserInfo={this.getUserInfoHandle}
          open-type='getUserInfo'>
          <Image src={scanIcon} />
        </Button> */}
        {/* <View className='scan-icon' onClick={this.clickHandle}>
          <Image src={scanIcon} />
        </View>
        <View className='scan-text'>马上扫一扫</View>
        <View className='scan-text'>使用多彩自助服务</View> */}
      </View>
    );
  }
}

export default Index as ComponentClass<PageOwnProps, PageState>;
